import { createComputed, onCleanup } from "solid-js";
import { a as addListener, d as getLineBefore, f as getLineEnd, s as scrollToEl } from "../index-CFdThse2.js";
import { template } from "solid-js/web";
const cursorTemplate = template(
  "<div style=position:absolute;top:0;opacity:0;padding:inherit> <span><span></span> "
);
const cursorPosition = () => (editor) => {
  let prevBefore = " ";
  let prevAfter = " ";
  const container = cursorTemplate();
  const [before, span] = container.childNodes;
  const [cursor, after] = span.childNodes;
  const scrollIntoView = () => scrollToEl(editor, cursor);
  const remove = addListener(editor, "input", (e) => {
    if (/history/.test(e.inputType)) scrollIntoView();
  });
  createComputed(() => {
    let selection = editor.selection();
    let value = editor.value;
    let position = selection[selection[2] < "f" ? 0 : 1];
    let newBefore = getLineBefore(value, position);
    let newAfter = value.slice(position, getLineEnd(value, position));
    let index = editor.activeLine;
    let activeLine = editor.lines[index];
    if (index) {
      if (!newBefore && !newAfter) newAfter = " ";
      if (prevBefore != newBefore) before.data = prevBefore = newBefore;
      if (prevAfter != newAfter) after.data = prevAfter = newAfter;
      if (container.parentNode != activeLine) activeLine.prepend(container);
    }
  });
  onCleanup(() => {
    delete editor.extensions.cursor;
    container.remove();
    remove();
  });
  editor.extensions.cursor = {
    scrollIntoView,
    getPosition() {
      const rect1 = cursor.getBoundingClientRect();
      const rect2 = editor.lines[0].getBoundingClientRect();
      return {
        top: rect1.y - rect2.y,
        bottom: rect2.bottom - rect1.bottom,
        left: rect1.x - rect2.x,
        right: rect2.right - rect1.x,
        height: rect1.height
      };
    }
  };
};
export {
  cursorPosition
};
//# sourceMappingURL=cursor.js.map
