import { r as regexEscape, a as addListener, s as scrollToEl, i as insertText } from "./index-CFdThse2.js";
import { template } from "solid-js/web";
import { onCleanup } from "solid-js";
const containerT = template(
  '<div style="color:#0000;display:none;contain:strict;padding:0 var(--_pse) 0 var(--padding-left)" aria-hidden=true>'
);
const matchTemplate = template("<span> ");
const testBoundary = (str, position, pattern = /[_\p{N}\p{L}]{2}/u) => {
  if (!position) return false;
  return pattern.test(
    str.slice(
      position - (str.codePointAt(position - 2) > 65535 ? 2 : 1),
      position + (str.codePointAt(position) > 65535 ? 2 : 1)
    )
  );
};
const createSearchAPI = (editor) => {
  const nodes = [new Text()];
  const nodeValues = [];
  const container = containerT();
  const matchPositions = [];
  const stopSearch = () => {
    if (matchPositions[0]) {
      matchPositions.length = 0;
      container.style.display = "none";
    }
  };
  let regex;
  let nodeCount = 0;
  return {
    search(str, caseSensitive, wholeWord, useRegExp, selection, filter, pattern) {
      if (!str) return stopSearch();
      if (!useRegExp) str = regexEscape(str);
      const value = editor.value;
      const searchStr = selection ? value.slice(...selection) : value;
      const offset = selection ? selection[0] : 0;
      let match;
      let l;
      let index;
      let i = 0;
      try {
        regex = RegExp(str, `gum${caseSensitive ? "" : "i"}`);
        while (match = regex.exec(searchStr)) {
          l = match[0].length;
          index = match.index + offset;
          if (!l) regex.lastIndex += value.codePointAt(index) > 65535 ? 2 : 1;
          if (wholeWord && (testBoundary(value, index, pattern) || testBoundary(value, index + l, pattern)))
            continue;
          if (!filter || filter(index, index + l)) matchPositions[i++] = [index, index + l];
        }
      } catch (e) {
        stopSearch();
        return e.message;
      }
      if (i) {
        matchPositions.length = i;
        l = Math.min(i * 2, 2e4);
        for (i = nodes.length; i <= l; ) {
          nodes[i++] = matchTemplate();
          nodes[i++] = new Text();
        }
        for (i = nodeCount - 1; i > l; ) nodes[i--].remove();
        if (nodeCount <= l) container.append(...nodes.slice(nodeCount, l + 1));
        let prevEnd = 0;
        for (i = 0; i < l; ++i) {
          const [start, end] = matchPositions[i / 2];
          const before = value.slice(prevEnd, start);
          const match2 = value.slice(start, prevEnd = end);
          if (before != nodeValues[i]) nodes[i].data = nodeValues[i] = before;
          if (match2 != nodeValues[++i]) nodes[i].firstChild.data = nodeValues[i] = match2;
        }
        nodes[l].data = nodeValues[l] = value.slice(prevEnd);
        container.style.display = "";
        nodeCount = l + 1;
      } else stopSearch();
    },
    container,
    get regex() {
      return regex;
    },
    matches: matchPositions,
    stopSearch
  };
};
const createReplaceAPI = (editor) => {
  const getSelection = editor.getSelection;
  const search = createSearchAPI(editor);
  const container = search.container;
  const matches = search.matches;
  const closest = () => {
    const caretPos = getSelection()[0];
    const l = matches.length;
    for (let i = l; i; ) {
      if (caretPos >= matches[--i][1]) return (i + (matches[i][0] < caretPos)) % l;
    }
    return l ? 0 : -1;
  };
  const toggleClasses = () => {
    currentLine?.classList.toggle("match-highlight");
    currentMatch?.classList.toggle("match");
  };
  const removeSelection = () => {
    if (hasSelected) {
      toggleClasses();
      hasSelected = false;
    }
  };
  const removeListener = addListener(editor, "focus", removeSelection);
  let currentLine;
  let currentMatch;
  let hasSelected = false;
  onCleanup(() => {
    removeListener();
    removeSelection();
  });
  return Object.assign(search, {
    next() {
      const cursor = getSelection()[1];
      const l = matches.length;
      for (let i = 0, match; i < l; i++) {
        match = matches[i];
        if (match[0] - (match[0] == match[1]) >= cursor) return i;
      }
      return l ? 0 : -1;
    },
    prev() {
      const cursor = getSelection()[0];
      const l = matches.length;
      for (let i = l, match; i; ) {
        match = matches[--i];
        if (match[1] + (match[0] == match[1]) <= cursor) return i;
      }
      return l - 1;
    },
    closest,
    selectMatch(index, scrollPadding) {
      removeSelection();
      if (matches[index]) {
        editor.setSelection(...matches[index]);
        currentLine = editor.lines[editor.activeLine];
        currentMatch = container.children[index];
        hasSelected = true;
        toggleClasses();
        if (currentMatch) {
          scrollToEl(editor, currentMatch, scrollPadding);
        }
      }
    },
    replace(str) {
      if (matches[0]) {
        let index = closest();
        let [start, end] = matches[index];
        let [caretStart, caretEnd] = getSelection();
        let notSelected = start != caretStart || end != caretEnd;
        if (notSelected) return index;
        if (editor.value.slice(start, end) == str) return matches[++index] ? index : 0;
        return insertText(editor, str);
      }
    },
    replaceAll(str) {
      if (!matches[0]) return;
      let value = editor.value;
      let [start, end] = getSelection();
      let newLen = str.length;
      let newStart = start;
      let newEnd = end;
      let newValue = "";
      let l = matches.length;
      for (let i = 0; i < l; i++) {
        const [matchStart, matchEnd] = matches[i];
        const lengthDiff = newLen - matchEnd + matchStart;
        const move = (pos) => matchStart > pos ? 0 : pos >= matchEnd ? lengthDiff : lengthDiff < 0 && pos > matchStart + newLen ? newLen + matchStart - pos : 0;
        newEnd += move(end);
        newStart += move(start);
        newValue += i ? value.slice(matches[i - 1][1], matchStart) + str : str;
      }
      insertText(editor, newValue, matches[0][0], matches[l - 1][1], newStart, newEnd);
    }
  });
};
export {
  createReplaceAPI as a,
  createSearchAPI as c
};
//# sourceMappingURL=replace-DcEWCntE.js.map
